#!/bin/sh
#
# Simple shell script to split VariCAD psotscript batch plot file 
# to single PS files and convert them to PDF
#
#    (c) Gerhard Reithofer, Techn. EDV Reithofer, 2010
#       gerhard.reithofer@tech.edv.co.at
#
#    This program is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your 
#    option) any later version.
#
#    This program is distributed in the hope that it will be useful, but
#    WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Known bugs and restrictions: 
#    Works only with standard formats a0 .. a4
#
# History:
#    20101103: 1st released version 0.1
#    20101103: paper rotation changed - version 0.2
#    20101106: checking for correct no. of plot files - version 0.3
#

INPFILE="$1"
PARFILE="$2"

PREFIX="psfile"
ASCRIPT=$(basename $0)
SC_VERS="0.3"
GS_EXEC="gs"

echo "$ASCRIPT $SC_VERS - PDF file creation from VariCAD batch plot file."
echo "  (c) Gerhard Reithofer, Techn. EDV Reithofer, 2010-11"
 
if [ -z "$INPFILE" ]; then 
  echo "  mailto:gerhard.reithofer@tech.edv.co.at"
  echo "  http://www.tech-edv.co.at"
  echo ""
  echo "Usage: $ASCRIPT plotfile [parmfile]"
  echo ""
  echo "Example: $ASCRIPT VariCAD_batch_print.prn Proj4711.bpr"
  echo " reads plot data from postscript file 'VariCAD_batch_print.prn'"
  echo " and filenames from VariCAD Plot batch parameter file 'Proj4711.bpr'"
  echo ""
  echo " If no batch plot parameter file is given, als files are named:"
  echo "        ${PREFIX}_xxx.pdf"
  echo " where xxx is a running number beginning from 0."
  echo " If a mathcing batch plot parameter file is specified as 2nd parameter"
  echo " all pdf files are renamed according the VariCAD drawing file names."
  echo ""
  echo " All result files and temporary files are created in current directory."
  echo " All files are overwritten if a file with same name exists in curr. dir."
  echo " All files with names '${PREFIX}_*.ps' in current directory are deleted."
  echo ""
  echo " Currently only standard formats a0 .. a4 are supported."
  echo " It requires an installed ghostscript interpreter."
  exit 1
fi

if [ ! -f "$INPFILE" ]; then 
  echo "File '$INPFILE' does not exist"
  exit 2
fi

###
### Split multiple postscriptfiles into single files
### with one page per file.
split_files()
{ # $1 .. batch plot postscript file
###
  awk -vname="$PREFIX" 'BEGIN {
    fcnt = outp = 0;
    outf = name "_" fcnt ".ps";
  }
  function make_PDF(file) {
    print "Postscript file " file " created.";
  }
  {
    if (substr($0,1,1) != "%") outp=1;
    if ($0 != "%!PS-Adobe-1.0") {
      print $0>>outf;
      next;
    } 
    close(outf);
    if (outp) {
      make_PDF(outf);
      outf = name "_" ++fcnt ".ps";
    } else {
      print "(Re)creating file " outf;
      print $0>outf;
    }
  }
  END {
    make_PDF(outf);
  }' $1
}

###
### Extract bounding box from a postscripts file
###
read_size()
{ # $1 .. postscriptfile 
### $2 .. flag -g for graphic bbox, otherwise file-bbox (optinal)
###            -r rotate bbox (swap x/y) - not tested!
  ROTATE=0;
  BBOX_PAT='%%BoundingBox'
  while [ "$1" != "" ] ; do
    case "$1" in
      "-g") BBOX_PAT='%%HiResBoundingBox' ;;
      "-r") ROTATE=1;;
        * ) INFILE=$1;;
    esac
    shift 1
  done

  $GS_EXEC -dQUIET -dSAFER -dNOPAUSE -dBATCH -sDEVICE=bbox $INFILE 2>&1 | \
     grep "^$BBOX_PAT"|awk -vrotate=$ROTATE 'BEGIN {
    fmt[0] = 1189;
    fmt[1] = 841;
    fmt[2] = 594;
    fmt[3] = 420;
    fmt[4] = 297;
    fmt[5] = 210;
  }
  function to_mm(val) {
    return int(val*25.4/72)
  } 
  function get_fmt(fx,fy,tx,ty) {
    dx = to_mm(tx)-to_mm(fx);
    dy = to_mm(ty)-to_mm(fy);
    if (dx>dy) 
      format = "landscape";
    else
      format = "portrait";
    for (i=0;i<5;i++) 
      if ((fmt[i+1] == dx && fmt[i]   == dy)|| 
          (fmt[i]   == dx && fmt[i+1] == dy))
        return "a" i " " format " " tx-fx " " ty-fy;
    return dx "x" dy " " format " " tx-fx " " ty-fy;
  } 
  { 
    if (rotate) 
      print get_fmt($3,$2,$5,$4);
    else
      print get_fmt($2,$3,$4,$5);
  }'
}

###
### Create pdf file from postscript with correct bbox
### values from file (graphic only!), orientation 3 (landscape)
### and remove postscript file (if no error)
###
make_pdf()
{ # .. $1 ps input file 
###
  INFILE=$1
  OUTFIL=$(echo $INFILE|sed 's/\.ps$/.pdf/')
  P_SIZE=$(read_size $INFILE)
  FORMAT=$(echo $P_SIZE|awk '{print $1}')
  ORIENT=$(echo $P_SIZE|awk '{print $2}')
  PSIZ_X=$(echo $P_SIZE|awk '{print $3}')
  PSIZ_Y=$(echo $P_SIZE|awk '{print $4}')
  if [ -z $(echo $FORMAT|grep 'a[0-4]') ]
  then echo "Unsupported sheet size $FORMAT - must be a0 .. a4"; exit 4
  fi
  echo "Creating $FORMAT PDF file from $INFILE ..."
  $GS_EXEC -dQUIET -dSAFER -dNOPAUSE -dBATCH \
     -sDEVICE="pdfwrite"  -sPAPERSIZE="$FORMAT" -sOutputFile="$OUTFIL" \
     -c ".setpdfwrite" -c "<< /PageSize [$PSIZ_X $PSIZ_Y] /Orientation 0 >> setpagedevice"  \
     -f $INFILE

  if [ $? = 0 ]
  then rm $INFILE
  else echo "Error $?, Postscript file not deleted."
  fi
}


#############################################################
###
### remove all temprary psfile_* 
###
echo "Removing all ${PREFIX}_[0-9]*.ps files."
rm -f ${PREFIX}_[0-9]*.ps

###
### split psfile from VariCACD batch print to single ps files
###
split_files $INPFILE

###
### Create PDF from each PS
###
for PS in ${PREFIX}_[0-9]*.ps
do make_pdf $PS
done

###
### Check if batch plot parameter files exists?
###
if [ "$PARFILE" = "" ]
then echo "Plot parameter file not defined, files named ${PREFIX}_xxx.pdf"
     ls ${PREFIX}_[0-9]*.pdf
     exit 0
fi

###
### If yes, rename PDFs to DWB-names
###
if [ ! -r "$PARFILE" ]; 
  then echo "Plot parameter file '$PARILE' is not readable"
  exit 3
fi

NUM_DWB=$(grep '\.dwb$' $PARFILE |wc -l)
NUM_PDF=$(ls ${PREFIX}_[0-9]*.pdf|wc -w)

if [ "$NUM_DWB" -ne "$NUM_PDF" ] ; then
  echo "*** WARNING *** not all VariCAD files have been plotted!"
  echo "$NUM_PDF of $NUM_DWB VariCAD files plotted, please check if drawing geometry is present."
  echo "Unable to determine orginal file names, files named ${PREFIX}_xxx.pdf"
  exit 4
fi

COUNTER=0
grep '\.dwb' "$PARFILE" | while read DWB_FILE 
  do NAME_ONLY=$(basename $DWB_FILE)
     NFILENAME=$(echo $NAME_ONLY|sed 's/dwb$/pdf/')
     CPDF_NAME=${PREFIX}_$COUNTER.pdf
     echo "Renaming file $CPDF_NAME to $NFILENAME"
     mv $CPDF_NAME $NFILENAME
     COUNTER=$(expr $COUNTER + 1)
  done

###
#############################################################
